/**
 * TCS Dependency Map Renderer
 *
 * Copyright (c) 2005 TopCoder, Inc. All rights Reserved.
 */
package com.topcoder.rules.dependencymap.view;

import java.util.Map;
import java.util.HashMap;
import java.util.List;
import java.util.ArrayList;
import java.util.Collections;

/**
 * This class represents an object and it's dependants. The application can use this class to build the dependency
 * graph that will be rendered by the DependencyDiagram. Each node can have two attributes:
 *
 * 1) The label (aka title) that will appear on the shape in the diagram.
 * 2) A specific shape to use for this specific node. The type should exist in the corresponding shape factory.
 * If not found at rendering time, the common shape will appear instead.
 *
 * @author Pops
 * @author TCSDeveloper
 */
public class Dependency {

    /**
     * This represents the title of this dependency. The title will be used by the layout manager to assign text
     * to the eventual shape. The title will also be considered a unique identifier of the dependency (ie no other
     * dependency should have the same title) by this component since the text of the title appears on the shape
     * (and thus is used to uniquely identify it). This variable is set by the constructor, immutable (has been
     * marked final to enforce immutability), and is access by the getter method. This variable has been marked
     * final to enforce it's immutablility (again - reference only).
     */
    private final String title;

    /**
     * This represents a map of dependency's that this dependency is dependant upon. The map contains the dependant's
     * title as the key and the Dependency as the value. The dependants are stored in a hashmap to allow quick
     * retrievals by title (which is considered unique identifier). The map may be empty if this dependency is not
     * dependant upon anything. This variable is set at object construction time, is immutable (the reference, not
     * the contents) and can be accessed by the add/remove/getter methods. This variable has been marked final to
     * enforce it's immutablility (again - reference only).
     */
    private final Map dependants = new HashMap();

    /**
     * This represents the type of shape that should override the common shape that would have been used to render
     * this element. The type must exist in the corresponding ShapeFactory to be rendered. This value will be -1
     * if there is no override. This variable is set in the constructor, is immutable once set and is access by
     * the getter method.
     */
    private final int shapeOverride;

    /**
     * This constructs the dependency from the specified title with no dependants. This constructor simply calls
     * the "this(title, -1)" constructor.
     *
     * @param title The title for the dependency
     * @throws NullPointerException if title is null
     * @throws IllegalArgumentException if title is empty (trim'd)
     */
    public Dependency(String title) {
        this(title, -1);
    }

    /**
     * This constructs the Dependency from the specified title and allows for a shape override. This constructor
     * simply saves the parameters to the like named variables.
     *
     * @param title The title for the dependency
     * @param shapeOverride A shape type that will be used to override the normal one
     *
     * @throws NullPointerException if  title is null
     * @throws IllegalArgumentException if title is empty (trim'd)
     * @throws IllegalArgumentException if shapeOverride is 0 or < -1 (ie it must be positive or -1)
     */
    public Dependency(String title, int shapeOverride) {
        if (title == null) {
            throw new NullPointerException("title should not be null");
        }
        if (title.trim().length() == 0) {
            throw new IllegalArgumentException("title should not be empty.");
        }
        if (shapeOverride < -1 || shapeOverride == 0) {
            throw new IllegalArgumentException("shapeOverride should be either -1 or positive.");
        }
        this.title = title;
        this.shapeOverride = shapeOverride;
    }

    /**
     * Simply the getter method for the title.
     *
     * @return The title of this component
     */
    public String getTitle() {
        return this.title;
    }

    /**
     * Returns an immutable list of Dependency elements. This list will reflect the dependency elements that
     * this dependency is dependant upon. This list will never be null but may be empty if there are no dependants.
     * The list should reflect the values() of the map.
     *
     * @return An immutable list of dependants.
     */
    public List getDependants() {
        return Collections.unmodifiableList(new ArrayList(this.dependants.values()));
    }

    /**
     * This method will add a new dependant for this dependency. This method should put the dependency into
     * the dependants map using the title as the key and the dependency as the value. If the title already exists
     * in the map, the related dependency should be overlaid by the specified parameter.
     *
     * @param dependant The dependency that this dependency is dependant upon.
     * @throws NullPointerException if dependency is null or the dependency's title is null
     */
    public void addDependant(Dependency dependant) {
        if (dependant == null) {
            throw new NullPointerException("dependant should not be null.");
        }
        this.dependants.put(dependant.getTitle(), dependant);
    }

    /**
     * This method will remove a Dependency (as identified by it's title) from the list. This method should remove
     * from the map the dependency that is identified by the title. Please note that no attempt should be made to
     * verify that the dependency does in fact match the one removed. True should be returned if the title was found
     * and the corresponding dependency removed.
     *
     * @param dependant The dependency to remove
     * @return True if the dependency was found and removed
     * @throws NullPointerException if dependant is null
     */
    public boolean removeDependant(Dependency dependant) {
        if (dependant == null) {
            throw new NullPointerException("dependant should not be null.");
        }
        return this.dependants.remove(dependant.getTitle()) != null;
    }

    /**
     * Returns the type that identifies the shape that should be used instead of the 'common' shape for this
     * dependency. If -1 is returned, the 'common' type for the dependency should be used.
     *
     * @return A type that identifies the shape that should be used or null if the common type should be used
     *         or -1 if the shape was not overridden
     */
    public int getShapeOverride() {
        return this.shapeOverride;
    }

}
