/**
 * TCS Dependency Map Renderer
 *
 * Copyright (c) 2005 TopCoder, Inc. All rights Reserved.
 */
package com.topcoder.rules.dependencymap.view;

import java.awt.Point;
import java.awt.Insets;
import java.awt.FontMetrics;
import java.awt.Rectangle;
import java.awt.geom.GeneralPath;
import java.awt.geom.AffineTransform;
import java.util.Set;
import java.util.HashSet;
import java.util.Map;
import java.util.HashMap;
import java.util.List;
import java.util.ArrayList;
import java.util.Iterator;

/**
 * This class is an implementation of the LayoutManager that will layout the dependency diagram into a fixed
 * sized grid (where all columns are fixed to a single width and all rows are fixed to a single height).
 *
 * @author Pops
 * @author TCSDeveloper
 */
public class FixedGridLayoutManager implements LayoutManager {

    /**
     * This variable represents the extra padding that should appear between columns. This variable is only used
     * *between* columns and will not provide margins around the outer columns. This variable is set in the
     * constructor, is immutable once set and will be used by the layout mechanisms. This variable has been marked
     * final to enforce it's immutablility (again - reference only).
     */
    private final int colPadding;

    /**
     * This variable represents the extra padding that should appear between rows. This variable is only used
     * *between* rows and will not provide margins around the outer rows.  This variable is set in the constructor,
     * is immutable once set and will be used by the layout mechanisms.  This variable has been marked final to
     * enforce it's immutablility (again - reference only).
     */
    private final int rowPadding;

    /**
     * This variable represents the margins that should be maintained between the text and shape. This variable is
     * set in the constructor, is immutable once set and will be used by the layout mechanisms. This variable has
     * been marked final to enforce it's immutablility (again - reference only).
     */
    private final Insets textMargin;

    /**
     * Constructs the fixed grid layout manager using a 10 pixel margin between cols and rows. This constructor
     * simply calls "this(10,10)".
     */
    public FixedGridLayoutManager() {
        this(10, 10);
    }

    /**
     * Constructs the fixed grid layout using the specified row/col padding with a default text margin of 10
     * pixels around all edges. This constructor should simply execute 'this(colPadding, rowPadding,
     * new Insets(10,10,10,10))'.
     *
     * @param colPadding The padding between columns
     * @param rowPadding The padding between rows
     * @throws IllegalArgumentException if either colPadding or rowPadding is less than 0
     */
    public FixedGridLayoutManager(int colPadding, int rowPadding) {
        this(colPadding, rowPadding, new Insets(10, 10, 10, 10));
    }

    /**
     * Constructs the fixed grid layout manager using the specified padding for cols/rows and margin around text.
     * This constructor simply saves the parameters to the like named variables.
     *
     * @param colPadding The padding between columns
     * @param rowPadding The padding between rows
     * @param textMargin The margin between the text and the shape
     * @throws IllegalArgumentException if either colPadding or rowPadding is less than 0
     * @throws NullPointerException if textMargin is null
     * @throws IllegalArgumentException if any attribute (left, right, top, bottom) within textMargin is less than 0
     */
    public FixedGridLayoutManager(int colPadding, int rowPadding, Insets textMargin) {
        if (colPadding < 0) {
            throw new IllegalArgumentException("colPadding should not be negative.");
        }
        if (rowPadding < 0) {
            throw new IllegalArgumentException("rowPadding should not be negative.");
        }
        if (textMargin == null) {
            throw new NullPointerException("textMargin should not be null.");
        }
        if (textMargin.left < 0 || textMargin.right < 0 || textMargin.top < 0 || textMargin.bottom < 0) {
            throw new IllegalArgumentException("textMargin's attributes should not be negative");
        }
        this.colPadding = colPadding;
        this.rowPadding = rowPadding;
        this.textMargin = (Insets) textMargin.clone();
    }

    /**
     * Calculate the widths of each of the unseen dependencies with is referenced by a root dependency.
     *
     * @param dependency the root dependency
     * @return a mapping from dependency to widths (Integer)
     */
    private Map getWidths(Dependency dependency) {
        // Mapping from dependants to widths (number of cells)
        Map widths = new HashMap();
        // Visitation count for dependants (Integer)
        Map seen = new HashMap();
        // BFS queue
        List queue = new ArrayList();

        // Initialize the queue and visitation count by placing the root dependency.
        queue.add(dependency);
        seen.put(dependency, new Integer(1));

        // Iterate through the queue.
        for (int i = 0; i < queue.size(); ++i) {
            dependency = (Dependency) queue.get(i);

            for (Iterator itr = dependency.getDependants().iterator(); itr.hasNext();) {
                Dependency dependant = (Dependency) itr.next();

                // If the dependency is seen for the first time, we need to calculate its
                // width, otherwise it's a seen dependency and has constant width of 1.
                if (!seen.containsKey(dependant)) {
                    queue.add(dependant);
                    seen.put(dependant, new Integer(1));
                } else {
                    seen.put(dependant, new Integer(((Integer) seen.get(dependant)).intValue() + 1));
                }
            }
        }

        // Calculate the width in a backward order.
        for (int i = queue.size() - 1; i >= 0; --i) {
            dependency = (Dependency) queue.get(i);

            int width = 0;
            for (Iterator itr = dependency.getDependants().iterator(); itr.hasNext();) {
                Dependency dependant = (Dependency) itr.next();

                // If the dependency is an unseen dependency, its width should already been calculated.
                // Otherwise its width is constantly 1.
                if (seen.get(dependant).equals(new Integer(1))) {
                    width += ((Integer) widths.get(dependant)).intValue();
                } else {
                    ++width;
                    seen.put(dependant, new Integer(((Integer) seen.get(dependant)).intValue() - 1));
                }
            }

            // If width is even make it odd.
            if (width % 2 == 0) {
                ++width;
            }

            // Store the width into the mapping.
            widths.put(dependency, new Integer(width));
        }

        return widths;
    }

    /**
     * Paint a dependency.
     *
     * @param dependency the dependency to paint
     * @param shape the shape for outbound
     * @param width the width of a cell
     * @param height the height of a cell
     * @param arrowHeight the height of an arrow
     * @param x the x coord position
     * @param y the y coord position
     * @param graphics the graphics to paint onto
     */
    private void paintDependency(Dependency dependency, GeneralPath shape, int width, int height,
                                 int arrowHeight, int x, int y, RendererGraphics graphics) {
        // Calculate pixel offset.
        x *= width + this.colPadding;
        y *= height + arrowHeight + this.rowPadding;

        // Scale it to cell size and translate it to position.
        Rectangle rect = shape.getBounds();
        shape.transform(AffineTransform.getScaleInstance(((double) width) / rect.width,
                                                         ((double) height) / rect.height));
        shape.transform(AffineTransform.getTranslateInstance(x, y));

        // Draw the shape.
        graphics.draw(shape);

        // Retrieve font metrics and estimate text width.
        FontMetrics metrics = graphics.getFontMetrics(graphics.getFont());
        int myTextWidth = metrics.stringWidth(dependency.getTitle());

        // Draw the text to the center of the cell.
        graphics.drawString(dependency.getTitle(), x + this.textMargin.left +
                            (width - this.textMargin.left - this.textMargin.right - myTextWidth) / 2,
                                                   y + this.textMargin.top + 1 + metrics.getAscent());
    }

    /**
     * Paint an arrow.
     *
     * @param shape the shape for the arrow
     * @param x1 the x coord of the first dependency
     * @param x2 the x coord of the second dependency
     * @param y the y coord of the first dependency
     * @param width the width of a cell
     * @param height the height of a cell
     * @param graphics the graphics to paint onto
     */
    private void paintArrow(GeneralPath shape, int x1, int x2, int y,
                            int width, int height, RendererGraphics graphics) {
        Rectangle rect = shape.getBounds();
        // Calculate arrow position.
        float x = (float) ((x1 + x2) / 2.0);
        x *= width + this.colPadding;
        x += (width - rect.width) / 2;
        y *= height + rect.height + this.rowPadding;
        y += height + this.rowPadding / 2;

        // Rotate it to point the dependant and translate it to position.
        shape.transform(AffineTransform.getRotateInstance(
                Math.PI / 2 - Math.atan2(height + rect.height + this.rowPadding,
                                         (x1 - x2) * (width + this.colPadding)),
                rect.width / 2.0, rect.height / 2.0));
        shape.transform(AffineTransform.getTranslateInstance(x, y));

        // Draw the arrow.
        graphics.draw(shape);
    }

    /**
     * Paint the dependency hiearchy.
     *
     * @param dependency the root dependency
     * @param widths the mapping of the width of the dependencies
     * @param shapeFactory the shape factory to produce the shapes
     * @param graphics the graphics to paint onto
     */
    private void paint(Dependency dependency, Map widths, ShapeFactory shapeFactory, RendererGraphics graphics) {
        // Calculate the maximum text size.
        FontMetrics metrics = graphics.getFontMetrics(graphics.getFont());
        int textHeight = metrics.getHeight();
        int textWidth = 0;
        for (Iterator itr = widths.keySet().iterator(); itr.hasNext();) {
            int myTextWidth = metrics.stringWidth(((Dependency) itr.next()).getTitle());

            if (myTextWidth > textWidth) {
                textWidth = myTextWidth;
            }
        }
        int height = textHeight + this.textMargin.top + this.textMargin.bottom + 2;
        int width = textWidth + this.textMargin.left + this.textMargin.right + 2;
        int arrowHeight = shapeFactory.getShape(ShapeFactory.RELATIONSHIP_ARROW).getBounds().height;

        // Visitation set
        Set seen = new HashSet();
        // BFS queue
        List queue = new ArrayList();
        // Position of the dependency to render
        List positions = new ArrayList();
        // The position of the parent dependency
        List parentPos = new ArrayList();

        // Initialize by inserting the root dependency.
        seen.add(dependency);
        queue.add(dependency);
        positions.add(new Point(0, 0));
        parentPos.add(null);

        // Render through the queue.
        for (int i = 0; i < queue.size(); ++i) {
            dependency = (Dependency) queue.get(i);
            Point position = (Point) positions.get(i);

            // Calculate the dependants width to decide whether we need a padding cell.
            int dependantsWidth = 0;
            for (Iterator itr = dependency.getDependants().iterator(); itr.hasNext();) {
                Dependency dependant = (Dependency) itr.next();

                if (!seen.contains(dependant)) {
                    dependantsWidth += ((Integer) widths.get(dependant)).intValue();
                } else {
                    ++dependantsWidth;
                }
            }
            boolean padding = dependantsWidth % 2 == 0;

            // If parent dependency exists we make the relationship arrow.
            if (parentPos.get(i) != null) {
                int parent = ((Integer) parentPos.get(i)).intValue();
                paintArrow(shapeFactory.getShape(ShapeFactory.RELATIONSHIP_ARROW),
                           parent, position.x + dependantsWidth / 2, position.y - 1, width, height, graphics);
            }

            // Obtain shape used for this unseen shape.
            GeneralPath shape = null;
            if (dependency.getShapeOverride() != -1) {
                shape = shapeFactory.getShape(dependency.getShapeOverride());
            }
            if (shape == null) {
                shape = shapeFactory.getShape(ShapeFactory.UNSEEN_ELEMENT);
            }
            paintDependency(dependency, shape, width, height,
                            arrowHeight, position.x + dependantsWidth / 2, position.y, graphics);

            // Accumulate the width to find the position for each dependant.
            int cumulativeWidth = 0;
            for (Iterator itr = dependency.getDependants().iterator(); itr.hasNext();) {
                Dependency dependant = (Dependency) itr.next();

                // Check to see if we make the padding on this dependant.
                if (padding && cumulativeWidth * 2 >= dependantsWidth) {
                    padding = false;
                    ++cumulativeWidth;
                }

                if (!seen.contains(dependant)) {

                    // Push the unseen dependant into the queue.
                    seen.add(dependant);
                    queue.add(dependant);
                    positions.add(new Point(position.x + cumulativeWidth, position.y + 1));
                    parentPos.add(new Integer(position.x + dependantsWidth / 2));

                    cumulativeWidth += ((Integer) widths.get(dependant)).intValue();
                } else {
                    // Draw arrow for the seen dependant.
                    paintArrow(shapeFactory.getShape(ShapeFactory.RELATIONSHIP_ARROW),
                               position.x + dependantsWidth / 2, position.x + cumulativeWidth, position.y,
                               width, height, graphics);

                    // Obtain shape used for this seen shape.
                    shape = null;
                    if (dependant.getShapeOverride() != -1) {
                        shape = shapeFactory.getShape(dependant.getShapeOverride());
                    }
                    if (shape == null) {
                        shape = shapeFactory.getShape(ShapeFactory.SEEN_ELEMENT);
                    }

                    // Draw the dependant.
                    paintDependency(dependant, shape, width, height,
                                    arrowHeight, position.x + cumulativeWidth, position.y + 1, graphics);

                    ++cumulativeWidth;
                }
            }
        }
    }

    /**
     * Lays out and prints the dependency diagram. Since the algorithm to do this is beyond Poseidon's
     * capabilities - please review the component specifications for details.
     *
     * @param dependency The dependency to layout
     * @param shapeFactory The shape factory to use
     * @param graphics The graphics to print to
     * @throws NullPointerException if either dependency, shapeFactory or g2d is null
     */
    public void layout(Dependency dependency, ShapeFactory shapeFactory, RendererGraphics graphics) {
        if (dependency == null) {
            throw new NullPointerException("dependency should not be null.");
        }
        if (shapeFactory == null) {
            throw new NullPointerException("shapeFactory should not be null.");
        }
        if (graphics == null) {
            throw new NullPointerException("graphics should not be null.");
        }

        Map widths = getWidths(dependency);

        paint(dependency, widths, shapeFactory, graphics);
    }

}
