/**
 * TCS Dependency Map Renderer
 *
 * Copyright (c) 2005 TopCoder, Inc. All rights Reserved.
 */
package com.topcoder.rules.dependencymap.view;

import java.awt.Insets;

import junit.framework.TestCase;

/**
 * Unit test cases for FixedGridLayoutManager.
 *
 * Testing of FixedGridLayoutManager includes the accuracy testing of the layout algorithm,
 * to verify that each parameter works independently.
 *
 * @author TCSDeveloper
 */
public class FixedGridLayoutManagerUnitTests extends TestCase {

    /**
     * The filename to persist to.
     */
    private String filename = "test_files/unittests/unittest-";

    /**
     * FixedGridLayoutManager instance to test.
     */
    private FixedGridLayoutManager manager = null;

    /**
     * ShapeFactory instance to use.
     */
    private ShapeFactory factory = null;

    /**
     * RendererGraphics to use.
     */
    private RendererGraphics graphics = null;

    /**
     * Create new layout manager, shape factory and renderer graphics for each test.
     */
    protected void setUp() {
        manager = new FixedGridLayoutManager();
        factory = new ShapeFactory();        
    }

    /**
     * Remove the file and dispose the renderer graphics.
     */
    protected void tearDown() {
        if (graphics != null) 
    	{
        	graphics.dispose();
    	}
    }

    /**
     * Create instance.
     */
    public void testCreate() {
        manager = new FixedGridLayoutManager();
    }

    /**
     * Create instance with colPadding and rowPadding.
     */
    public void testCreate_ColPaddingRowPadding() {
        manager = new FixedGridLayoutManager(10, 10);
    }

    /**
     * Create instance with negative colPadding.
     */
    public void testCreate_NegativeColPadding() {
        try {
            new FixedGridLayoutManager(-1, 10);
            fail("Should have thrown IllegalArgumentException.");
        } catch (IllegalArgumentException iae) {
        }
    }

    /**
     * Create instance with negative rowPadding.
     */
    public void testCreate_NegativeRowPadding() {
        try {
            new FixedGridLayoutManager(10, -1);
            fail("Should have thrown IllegalArgumentException.");
        } catch (IllegalArgumentException iae) {
        }
    }

    /**
     * Create instance with colPadding, rowPadding and textMargin.
     */
    public void testCreate_ColPaddingRowPaddingTextMargin() {
        manager = new FixedGridLayoutManager(10, 10, new Insets(10, 10, 10, 10));
    }

    /**
     * Create instance with negative colPadding.
     */
    public void testCreate_NegativeColPadding2() {
        try {
            new FixedGridLayoutManager(-1, 10, new Insets(10, 10, 10, 10));
            fail("Should have thrown IllegalArgumentException.");
        } catch (IllegalArgumentException iae) {
        }
    }

    /**
     * Create instance with negative rowPadding.
     */
    public void testCreate_NegativeRowPadding2() {
        try {
            new FixedGridLayoutManager(10, -1, new Insets(10, 10, 10, 10));
            fail("Should have thrown IllegalArgumentException.");
        } catch (IllegalArgumentException iae) {
        }
    }

    /**
     * Create instance with null textMargin.
     */
    public void testCreate_NullTextMargin() {
        try {
            new FixedGridLayoutManager(10, 10, null);
            fail("Should have thrown NullPointerException.");
        } catch (NullPointerException npe) {
        }
    }

    /**
     * Create instance with negative top.
     */
    public void testCreate_NegativeTop() {
        try {
            new FixedGridLayoutManager(10, 10, new Insets(-1, 10, 10, 10));
            fail("Should have thrown IllegalArgumentException.");
        } catch (IllegalArgumentException iae) {
        }
    }

    /**
     * Create instance with negative left.
     */
    public void testCreate_NegativeLeft() {
        try {
            new FixedGridLayoutManager(10, 10, new Insets(10, -1, 10, 10));
            fail("Should have thrown IllegalArgumentException.");
        } catch (IllegalArgumentException iae) {
        }
    }

    /**
     * Create instance with negative bottom.
     */
    public void testCreate_NegativeBottom() {
        try {
            new FixedGridLayoutManager(10, 10, new Insets(10, 10, -1, 10));
            fail("Should have thrown IllegalArgumentException.");
        } catch (IllegalArgumentException iae) {
        }
    }

    /**
     * Create instance with negative right.
     */
    public void testCreate_NegativeRight() {
        try {
            new FixedGridLayoutManager(10, 10, new Insets(10, 10, 10, -1));
            fail("Should have thrown IllegalArgumentException.");
        } catch (IllegalArgumentException iae) {
        }
    }

    /**
     * Layout single dependency.
     *
     * @throws RendererException to JUnit.
     */
    public void testLayout_SingleDependency() throws RendererException {
        Dependency dependency = new Dependency("Dependency Map Renderer");
        
        graphics = new SVGGraphicsAdapter(filename + "single_dependency.svg");
        
        manager.layout(dependency, factory, graphics);
        graphics.complete();
    }

    /**
     * Layout dependency tree.
     *
     * @throws RendererException to JUnit.
     */
    public void testLayout_DependencyTree() throws RendererException {
        Dependency dependency = new Dependency("Dependency Map Renderer");
        dependency.addDependant(new Dependency("SVG Generator"));
        dependency.addDependant(new Dependency("Base Exception"));
        
        graphics = new SVGGraphicsAdapter(filename + "dependency_tree.svg");
        
        manager.layout(dependency, factory, graphics);
        graphics.complete();
    }

    /**
     * Layout seen dependency.
     *
     * @throws RendererException to JUnit.
     */
    public void testLayout_SeenDependency() throws RendererException {
        Dependency dependency = new Dependency("Dependency Map Renderer");
        dependency.addDependant(dependency);

        graphics = new SVGGraphicsAdapter(filename + "seen_dependency.svg");

        manager.layout(dependency, factory, graphics);
        graphics.complete();
    }

    /**
     * Layout shapeOverride.
     *
     * @throws RendererException to JUnit.
     */
    public void testLayout_ShapeOverride() throws RendererException {
        Dependency dependency = new Dependency("Dependency Map Renderer", ShapeFactory.SEEN_ELEMENT);
        
        graphics = new SVGGraphicsAdapter(filename + "shape_override.svg");
        
        manager.layout(dependency, factory, graphics);
        graphics.complete();
    }

    /**
     * Layout colPadding and rowPadding.
     *
     * @throws RendererException to JUnit.
     */
    public void testLayout_ColPaddingRowPadding() throws RendererException {
        manager = new FixedGridLayoutManager(0, 50);
        Dependency dependency = new Dependency("Dependency Map Renderer");
        dependency.addDependant(new Dependency("SVG Generator"));
        dependency.addDependant(new Dependency("Base Exception"));

        graphics = new SVGGraphicsAdapter(filename + "colpadding_rowpadding.svg");

        manager.layout(dependency, factory, graphics);
        graphics.complete();
    }

    /**
     * Layout textMargin.
     *
     * @throws RendererException to JUnit.
     */
    public void testLayout_TextMargin() throws RendererException {
        manager = new FixedGridLayoutManager(10, 10, new Insets(50, 0, 50, 0));
        Dependency dependency = new Dependency("Dependency Map Renderer");
        dependency.addDependant(new Dependency("SVG Generator"));
        dependency.addDependant(new Dependency("Base Exception"));

        graphics = new SVGGraphicsAdapter(filename + "text_margin.svg");

        manager.layout(dependency, factory, graphics);
        graphics.complete();
    }

    /**
     * Test layout() with null dependency.
     */
    public void testLayout_NullDependency() {
        try {

            graphics = new SVGGraphicsAdapter(filename + "fail_layout.svg");
            manager.layout(null, factory, graphics);
            fail("Should have thrown NullPointerException.");
        } catch (NullPointerException npe) {
        }
    }

    /**
     * Test layout() with null shape factory.
     */
    public void testLayout_NullShapeFactory() {
        try {
            graphics = new SVGGraphicsAdapter(filename + "fail_factory.svg");
            manager.layout(new Dependency("a"), null, graphics);
            fail("Should have thrown NullPointerException.");
        } catch (NullPointerException npe) {
        }
    }

    /**
     * Test layout() with null renderer graphics.
     */
    public void testLayout_NullRendererGraphics() {
        try {
            manager.layout(new Dependency("a"), factory, null);
            fail("Should have thrown NullPointerException.");
        } catch (NullPointerException npe) {
        }
    }

}
